@@ -33,6 +33,7 @@
 // KDE
 #include <klocalizedstring.h>
 #include <kwindowsystem.h>
+#include <kglobalaccel.h>
 
 // KMix
 #include "apps/kmix.h"
@@ -48,36 +49,53 @@
 #undef RESTORE_VOLUME_BUTTON
 
 
-ViewDockAreaPopup::ViewDockAreaPopup(QWidget* parent, const QString &id, ViewBase::ViewFlags vflags, const QString &guiProfileId,
-	KMixWindow *dockW) :
-	ViewBase(parent, id, {}, vflags, guiProfileId), _kmixMainWindow(dockW)
+ViewDockAreaPopup::ViewDockAreaPopup(QWidget* parent, const QString &id, ViewBase::ViewFlags vflags,
+				     const QString &guiProfileId, KMixWindow *dockW)
+	: ViewBase(parent, id, {}, vflags, guiProfileId),
+	  _kmixMainWindow(dockW)
 {
 	resetRefs();
 	setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum);
-	/*
-	 * Bug 206724:
-	 * Following are excerpts of trying to receive key events while this popup is open.
-	 * The best I could do with lots of hacks is to get the keyboard events after a mouse-click in the popup.
-	 * But such a solution is neither intuitive nor helpful - if one clicks, then usage of keyboard makes not much sense any longer. 
-	 * I finally gave up on fixing Bug 206724.
-	 */
-	 /*
-	releaseKeyboard();
+
+	// Original comments from cesken on commit 2bd1c4cb 12 Aug 2014:
+	//
+	//   Bug 206724:
+	//   Following are excerpts of trying to receive key events while
+	//   this popup is open.  The best I could do with lots of hacks is
+	//   to get the keyboard events after a mouse-click in the popup.
+	//   But such a solution is neither intuitive nor helpful - if one
+	//   clicks, then usage of keyboard makes not much sense any longer.
+	//   I finally gave up on fixing Bug 206724.
+	//
+	//   releaseKeyboard();
+	//   setFocusPolicy(Qt::StrongFocus);
+	//   setFocus(Qt::TabFocusReason);
+	//   releaseKeyboard();
+	//   mainWindowButton->setFocusPolicy(Qt::StrongFocus);
+	//
+	//   Also implemented the following "event handlers", but they
+	//   do not show any signs of key presses:
+	//
+	//   keyPressEvent()
+	//   x11Event()
+	//   eventFilter()
+	//
+	// However, that applied to KDE4/Qt4.  it does now seem to be possible
+	// to receive key events on the popup, by doing setFocusPolicy() and
+	// setFocus() below.  setFocus() needs to be called on an idle timer,
+	// if it is not called or is called directly there is the very strange
+	// effect that the popup does not receive key events until one of the
+	// up/down arrow keys is first pressed;  the first key disappears but
+	// then all subsequent events for all keys are received.
+	//
 	setFocusPolicy(Qt::StrongFocus);
-	setFocus(Qt::TabFocusReason);
-	releaseKeyboard();
-	mainWindowButton->setFocusPolicy(Qt::StrongFocus);
-	
-	Also implemented the following "event handlers", but they do not show any signs of key presses:
-    keyPressEvent()
-    x11Event()
-    eventFilter()
-    */
-	foreach ( Mixer* mixer, Mixer::mixers() )
+	QTimer::singleShot(0, this, [this]() { setFocus(Qt::OtherFocusReason); });
+
+	// Adding all mixers, as we potentially want to show all master controls.
+	// The list will be redone in initLayout() with the actual Mixer instances to use.
+	for (Mixer *mixer : Mixer::mixers())
 	{
-		// Adding all mixers, as we potentially want to show all master controls
 		addMixer(mixer);
-		// The list will be redone in initLayout() with the actual Mixer instances to use
 	}
 
 	restoreVolumeIcon = QIcon::fromTheme(QLatin1String("quickopen-file"));
@@ -99,6 +117,64 @@ ViewDockAreaPopup::~ViewDockAreaPopup()
 }
 
 
+void ViewDockAreaPopup::keyPressEvent(QKeyEvent *ev)
+{
+	// Key events are received here, after the widget has been set up for
+	// focus in the constructor above.  However, this event handler receives
+	// all key events, including global ones - this is presumably because
+	// KMixDockWidget wraps the popup volume control in a menu which grabs
+	// the keyboard while shown.  Releasing the grab closes the menu and
+	// hence the popup, so the grab has to be maintained.
+	//
+	// Handle the small subset of key events that we are interested in:
+	// the up/down arrow keys for a volume step, Page Up/Down for a
+	// double step, and Space or M to toggle mute.  Since we also receive
+	// global shortcuts, check for the configured KMix global shortcuts
+	// and act on them.  Unfortunately, as is the case with any menu when
+	// it is active, all other keys are grabbed and other global shortcuts
+	// will not work.  The standard QWidget::keyPressEvent() handler will
+	// close the popup on Escape.
+
+	KActionCollection *ac = _kmixMainWindow->actionCollection();
+	QAction *actUp = ac->action("increase_volume");
+	QAction *actDown = ac->action("decrease_volume");
+	QAction *actMute = ac->action("mute");
+
+	int key = ev->key();
+	const QKeySequence seq(key);
+	// KGlobalAccel::shortcut() is safe if called with a null pointer.
+	if (KGlobalAccel::self()->shortcut(actUp).contains(seq)) key = Qt::Key_Up;
+	if (KGlobalAccel::self()->shortcut(actDown).contains(seq)) key = Qt::Key_Down;
+	if (KGlobalAccel::self()->shortcut(actMute).contains(seq)) key = Qt::Key_Space;
+
+	switch (key)
+	{
+case Qt::Key_PageUp:
+            if (actUp!=nullptr) actUp->trigger();	// double step up
+	    Q_FALLTHROUGH();
+
+case Qt::Key_Up:
+            if (actUp!=nullptr) actUp->trigger();	// single step up
+            return;
+
+case Qt::Key_PageDown:
+            if (actDown!=nullptr) actDown->trigger();	// double step down
+	    Q_FALLTHROUGH();
+
+case Qt::Key_Down:
+            if (actDown!=nullptr) actDown->trigger();	// single step down
+            return;
+
+case Qt::Key_M:
+case Qt::Key_Space:
+            if (actMute!=nullptr) actMute->trigger();	// toggle mute
+            return;
+	}
+
+	QWidget::keyPressEvent(ev);			// handle others, including ESC
+}
+
+
 void ViewDockAreaPopup::controlsChange(ControlManager::ChangeType changeType)
 {
   switch (changeType)
@@ -137,24 +213,24 @@ void ViewDockAreaPopup::configurationUpdate()
 
 void ViewDockAreaPopup::resetRefs()
 {
-	seperatorBetweenMastersAndStreams = 0;
+	seperatorBetweenMastersAndStreams = nullptr;
 	separatorBetweenMastersAndStreamsInserted = false;
 	separatorBetweenMastersAndStreamsRequired = false;
-	configureViewButton = 0;
-	restoreVolumeButton1 = 0;
-	restoreVolumeButton2 = 0;
-	restoreVolumeButton3 = 0;
-	restoreVolumeButton4 = 0;
-	mainWindowButton = 0;
-	optionsLayout = 0;
-	_layoutMDW = 0;
+	configureViewButton = nullptr;
+	restoreVolumeButton1 = nullptr;
+	restoreVolumeButton2 = nullptr;
+	restoreVolumeButton3 = nullptr;
+	restoreVolumeButton4 = nullptr;
+	mainWindowButton = nullptr;
+	optionsLayout = nullptr;
+	_layoutMDW = nullptr;
 }
 
 void ViewDockAreaPopup::initLayout()
 {
 	resetMdws();
 
-	if (optionsLayout != 0)
+	if (optionsLayout!=nullptr)
 	{
 		QLayoutItem *li2;
 		while ((li2 = optionsLayout->takeAt(0)))
@@ -162,7 +238,7 @@ void ViewDockAreaPopup::initLayout()
 	}
 // Hint : optionsLayout itself is deleted when "delete _layoutMDW" cascades down
 
-	if (_layoutMDW != 0)
+	if (_layoutMDW!=nullptr)
 	{
 		QLayoutItem *li;
 		while ((li = _layoutMDW->takeAt(0)))
@@ -179,7 +255,7 @@ Application: KMix (kmix), signal: Segmentation fault
 #8  0x00007f9c9d425700 in ViewBase::createDeviceWidgets (this=0x1272b60) at /home/chris/workspace/kmix-git-trunk/gui/viewbase.cpp:137
 #9  0x00007f9c9d42845b in ViewDockAreaPopup::controlsChange (this=0x1272b60, changeType=2) at /home/chris/workspace/kmix-git-trunk/gui/viewdockareapopup.cpp:91
     */
-//   if ( optionsLayout != 0 )
+//   if ( optionsLayout != nullptr )
 //   {
 //     QLayoutItem *li2;
 //     while ( ( li2 = optionsLayout->takeAt(0) ) ) // strangely enough, it crashes here
@@ -208,7 +284,7 @@ Application: KMix (kmix), signal: Segmentation fault
 	 *             of this  ViewDockAreaPopup
 	 *             (Hint: it might have been 0 already. Nowadays it is definitely, see #resetRefs())
 	 */
-	delete layout(); // BKO 299754
+	delete layout(); // Bug 299754
 	_layoutMDW = new QGridLayout(this);
 	_layoutMDW->setSpacing(DialogBase::verticalSpacing());
 	// Review #121166: Add some space over device icons, otherwise they may hit window border
@@ -218,13 +294,14 @@ Application: KMix (kmix), signal: Segmentation fault
 	_layoutMDW->setObjectName(QLatin1String("KmixPopupLayout"));
 	setLayout(_layoutMDW);
 
-	// Adding all mixers, as we potentially want to show all master controls. Due to hotplugging
-	// we have to redo the list on each initLayout() (instead of setting it once in the Constructor)
-	_mixers.clear();
+	// Adding all mixers, as we potentially want to show all master controls.
+	// Due to hotplugging we have to redo the list on each initLayout() instead of
+	// setting it once in the constructor.
+	clearMixers();
 
 	QSet<QString> preferredMixersForSoundmenu = GlobalConfig::instance().getMixersForSoundmenu();
 	//qCDebug(KMIX_LOG) << "Launch with " << preferredMixersForSoundmenu;
-	foreach ( Mixer* mixer, Mixer::mixers() )
+	for (Mixer *mixer : qAsConst(Mixer::mixers()))
 	{
 		bool useMixer = preferredMixersForSoundmenu.isEmpty() || preferredMixersForSoundmenu.contains(mixer->id());
 		if (useMixer) addMixer(mixer);
@@ -233,9 +310,9 @@ Application: KMix (kmix), signal: Segmentation fault
 	// The following loop is for the case when everything gets filtered out. We "reset" to show everything then.
 	// Hint: Filtering everything out can only be an "accident", e.g. when restarting KMix with changed hardware or
 	// backends.
-	if ( _mixers.isEmpty() )
+	if (getMixers().isEmpty())
 	{
-		foreach ( Mixer* mixer, Mixer::mixers() )
+		for (Mixer *mixer : qAsConst(Mixer::mixers()))
 		{
 			addMixer(mixer);
 		}
@@ -258,7 +335,7 @@ Application: KMix (kmix), signal: Segmentation fault
 	//
 	// Maybe need a configuration option?
 
-	foreach (const Mixer *mixer, _mixers)
+	for (const Mixer *mixer : qAsConst(getMixers()))
 	{
 		//qCDebug(KMIX_LOG) << "ADD? mixerId=" << mixer->id();
 		// Get the configured master control for the mixer.
@@ -279,17 +356,17 @@ Application: KMix (kmix), signal: Segmentation fault
 			if (dockMD->playbackVolume().hasVolume() || dockMD->captureVolume().hasVolume())
 			{
 				//qCDebug(KMIX_LOG) << "ADD? mixerId=" << mixer->id() << ", md=" << dockMD->id() << ": YES";
-				_mixSet.append(dockMD);
+				addToMixSet(dockMD);
 			}
 		}
 	} // loop over all cards
 
 	// Finally add all application streams
-	foreach (const Mixer *mixer, _mixers)
+	for (const Mixer *mixer : qAsConst(getMixers()))
 	{
-		foreach (shared_ptr<MixDevice> md, mixer->getMixSet())
+		for (shared_ptr<MixDevice> md : qAsConst(mixer->getMixSet()))
 		{
-			if (md->isApplicationStream()) _mixSet.append(md);
+			if (md->isApplicationStream()) addToMixSet(md);
 		}
 	}
 }
@@ -417,21 +494,12 @@ void ViewDockAreaPopup::refreshVolumeLevels()
 
 void ViewDockAreaPopup::configureView()
 {
-//    Q_ASSERT( !pulseaudioPresent() );
-
-//    QSet<QString> currentlyActiveMixersInDockArea;
-//	foreach ( Mixer* mixer, _mixers )
-//	{
-//		currentlyActiveMixersInDockArea.insert(mixer->id());
-//	}
-
-	KMixPrefDlg* prefDlg = KMixPrefDlg::getInstance();
-	//prefDlg->setActiveMixersInDock(currentlyActiveMixersInDockArea);
+	KMixPrefDlg *prefDlg = KMixPrefDlg::getInstance();
 	prefDlg->switchToPage(KMixPrefDlg::PrefSoundMenu);
 }
 
 /**
- * This gets activated whne a user clicks the "Mixer" PushButton in this popup.
+ * This gets activated when a user clicks the "Mixer" PushButton in this popup.
  */
 void ViewDockAreaPopup::showPanelSlot()
 {
